package com.iconplus.smartproc.service.jenispengadaan;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.JenisPengadaanView;
import com.iconplus.smartproc.model.request.JenisPengadaanRequest;
import com.iconplus.smartproc.model.response.GetListJenisPengadaanResponse;
import com.iconplus.smartproc.model.response.JenisPengadaanResponse;
import com.iconplus.smartproc.repository.JenisPengadaanRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListJenisPengadaanService implements BaseService<JenisPengadaanRequest, GetListJenisPengadaanResponse> {

    private JenisPengadaanRepository jenisPengadaanRepository;
    public GetListJenisPengadaanService(JenisPengadaanRepository jenisPengadaanRepository) {
        this.jenisPengadaanRepository = jenisPengadaanRepository;
    }

    @Override
    public GetListJenisPengadaanResponse execute(JenisPengadaanRequest input) {
        validateRequest(input);
        List<JenisPengadaanResponse> jenisPengadaanResponseList = new ArrayList<>();
        var jenisPengadaanViews = jenisPengadaanRepository.getListJenisPengadaan(input.getSearch(), input.getPageable());

        for (JenisPengadaanView jenisPengadaanView : jenisPengadaanViews) {
            JenisPengadaanResponse jenisPengadaanResponse = JenisPengadaanResponse.builder()
                    .id(jenisPengadaanView.getId())
                    .jenisPengadaan(jenisPengadaanView.getJenisPengadaan())
                    .keterangan(jenisPengadaanView.getKeterangan())
                    .isActive(jenisPengadaanView.getIsActive())
                    .build();
            jenisPengadaanResponseList.add(jenisPengadaanResponse);
        }

        return GetListJenisPengadaanResponse.builder()
                .data(jenisPengadaanResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(jenisPengadaanViews.getTotalPages())
                        .totalRecords(jenisPengadaanViews.getTotalElements())
                        .isFirstPage(jenisPengadaanViews.isFirst())
                        .isLastPage(jenisPengadaanViews.isLast())
                        .build())
                .build();

    }

    private void validateRequest(JenisPengadaanRequest input) {
        if (StringUtils.isNotBlank(input.getSearch())) {
            input.setSearch('%'+ input.getSearch().toUpperCase()+'%');
        }
    }
}
